<?php

declare(strict_types=1);

namespace Drupal\mcp_client\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\mcp_client\McpServerInterface;

/**
 * Defines the MCP Server entity type.
 *
 * @ConfigEntityType(
 *   id = "mcp_server",
 *   label = @Translation("MCP Server"),
 *   label_collection = @Translation("MCP Servers"),
 *   label_singular = @Translation("MCP Server"),
 *   label_plural = @Translation("MCP Servers"),
 *   label_count = @PluralTranslation(
 *     singular = "@count MCP Server",
 *     plural = "@count MCP Servers",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\mcp_client\McpServerListBuilder",
 *     "form" = {
 *       "add" = "Drupal\mcp_client\Form\McpServerForm",
 *       "edit" = "Drupal\mcp_client\Form\McpServerForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *   },
 *   config_prefix = "mcp_server",
 *   admin_permission = "administer mcp_server",
 *   links = {
 *     "collection" = "/admin/structure/mcp-server",
 *     "add-form" = "/admin/structure/mcp-server/add",
 *     "edit-form" = "/admin/structure/mcp-server/{mcp_server}",
 *     "delete-form" = "/admin/structure/mcp-server/{mcp_server}/delete",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "description",
 *     "endpoint",
 *     "tools",
 *     "enabled_tools",
 *   },
 * )
 */
final class McpServer extends ConfigEntityBase implements McpServerInterface {

  /**
   * The example ID.
   */
  protected string $id;

  /**
   * The example label.
   */
  protected string $label;

  /**
   * The example description.
   */
  protected string $description;

  /**
   * The MCP server endpoint.
   */
  protected string $endpoint;

  /**
   * The available tools.
   */
  protected array $tools = [];

  /**
   * The enabled tools.
   */
  protected array $enabled_tools = [];

}
