<?php

namespace Drupal\mcp_client\Plugin\AiFunctionCall\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a base action for each entity type with specific interfaces.
 */
class McpPluginDeriver extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * Constructs a new McpPluginDervier object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    if (empty($this->derivatives)) {
      $definitions = [];
      // Load all MCP Servers.
      $mcp_servers = $this->entityTypeManager->getStorage('mcp_server')->loadMultiple();
      if (empty($mcp_servers)) {
        return [];
      }
      foreach ($mcp_servers as $mcp_server) {
        // Only enabled ones.
        if (!$mcp_server->get('status')) {
          continue;
        }
        // Get all tools.
        $tools = $mcp_server->get('tools');
        if (empty($tools)) {
          continue;
        }
        // Get all enabled tools.
        $enabled_tools = $mcp_server->get('enabled_tools');
        if (empty($enabled_tools)) {
          continue;
        }
        // Loop through them.
        foreach ($enabled_tools as $tool) {
          if (isset($tools[$tool])) {
            $definition = $base_plugin_definition;
            $id = $mcp_server->get('id') . ':' . $tools[$tool]['name'];
            $definition['id'] = 'mcp_plugin:' . $id;
            $definition['name'] = $tools[$tool]['name'];
            $definition['group'] = 'mcp_client';
            $definition['function_name'] = str_replace(':', '__', $definition['id']);
            $definition['description'] = $tools[$tool]['description'];
            $context_definitions = [];
            foreach ($tools[$tool]['inputSchema']['properties'] as $key => $info) {
              // Temporary fix for the context definition.
              $type = $info['type'];
              if ($info['type'] == 'number') {
                $type = 'float';
              }
              if ($info['type'] == 'object') {
                // Do not allow objects.
                $type = 'string';
              }
              $context_definitions[$key] = new ContextDefinition($type, $key, $info['required'] ?? FALSE, FALSE, $info['description'], $info['default_value'] ?? NULL, []);
            }
            $definition['context_definitions'] = $context_definitions;
            $definitions[$id] = $definition;
          }
        }
        $this->derivatives = $definitions;
      }
    }
    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
